const vscodeApi = acquireVsCodeApi();
const events = ['explain', 'comment' ,'test', 'optimization', 'fix'];
const i18n = window.ChatParams.i18n || {};
const eventsMap = {
    explain: i18n['cmdk.dropdown.explain'],
    comment: i18n['cmdk.dropdown.comment'],
    test: i18n['cmdk.dropdown.test'],
    optimization: i18n['cmdk.dropdown.optimization'],
    fix: i18n['cmdk.dropdown.fix']
}
let loading = false;
let isOnComposition = false;
function handleLoading (values) {
    const $loading = document.querySelector('.inscode-ai-optimizate-loading');
    const $generate = document.querySelector('#generate');
    loading = values.loading
    if (loading) {
        $loading.style.display = 'block';
        $textarea.style.display = 'none';
        $textareaHide.style.display = 'block';
        $generate.innerHTML = i18n['cmdk.btn.stop'];
        handleUpdateActionVisible(false)
    } else {
        $loading.style.display = 'none';
        $generate.innerHTML = i18n['generate.btn'];
        $textarea.style.display = 'block';
        $textareaHide.style.display = 'none';
        $generate.style.cursor = 'pointer';
        handleFocusLast();
    }
    if (values.showResult) {
        handleUpdateActionVisible(true)
    } 
}
function handleUpdateActionVisible (bool) {
    const $actions = document.querySelectorAll('.inscode-ai-optimizate-btn');
    const types = ['continue', 'refuse', 'accept'];
    $actions.forEach(item => {
        const type = item.getAttribute('data-type');
        if (types.includes(type)) {
            item.style.display = bool ? 'block' : 'none';
        } else {
            item.style.display = bool ? 'none' : 'block';
        }
    })
}
function getSubmitPrefix(text) {
    const findPrefix = events.find(item => text.indexOf(`/${item}`) === 0)
    const query = { type: '', input: text }
    if (findPrefix) {
        query.type = findPrefix;
        const preText = `/${findPrefix}`;
        query.input = text.slice(preText.length, text.length);
    }
    return query
}
function handleSubmit () {
    const value = $textarea.innerText;
    if (value.trim() === '' || loading) return;
    const query = getSubmitPrefix(value)
    if (query.input.trim()) {
        window.ChatHistory.push(query.input)
        const len = window.ChatHistory.length
        if ( len > 10) {
            window.ChatHistory = window.ChatHistory.slice(len - 10, len)
        }
    }
    vscodeApi.postMessage({ 
        type: 'inscode.ai.editor.optimizate.dialog.submit',
        values: {
            ...query,
            cmdHistoryList: window.ChatHistory
        } 
    });
}
function handleClose () {
    vscodeApi.postMessage({ type: 'inscode.ai.editor.optimizate.dialog.close' });
}
function findPrefixTag (text) {
    return events.find(item => text === `/${item}`) 
}
function handleFocusLast () {
    $textarea.focus();
    const range = document.createRange();
    range.selectNodeContents($textarea);
    range.collapse(false);
    const selection = window.getSelection();
    selection.removeAllRanges();
    selection.addRange(range);
}
function handleUpdateTextareaTag (text) {
    const isTag = findPrefixTag(text);
    if (isTag) {
        $textarea.innerHTML = `<span class="inscode-ai-optimizate-tag" contenteditable="false">${text}</span> `;
        handleFocusLast();
    }
}
function handleUpdateGenerateStatus () {
    const $generate = document.querySelector('#generate');
    const text = $textarea.innerText;
    const baseClass = 'inscode-ai-optimizate-btn';
    if (text.trim() !== '' || loading) {
        $generate.className = `${baseClass} inscode-ai-optimizate-btn-primary`;
    } else {
        $generate.className = `${baseClass} inscode-ai-optimizate-btn-white`
    }
}
function handleRemoveEntry () {
    const value = $textarea.innerHTML;
    const n = `\n`;
    if (value.endsWith(n)) {
        $textarea.innerHTML = value.replace(/\n+$/, '');
        handleFocusLast();
    }
}
function storeTextareaHtml () {
    $textareaHide.innerHTML = $textarea.innerHTML;
    // const text = $textarea.innerText;
    // const { input, type } = getSubmitPrefix(text)
    // let html = type ? `<span class="inscode-ai-optimizate-tag">/${type}</span>` : '';
    // html += input
    // $textareaHide.innerHTML = html;
}
let filterEvents = [];
function handleUpdateInput(e) {
    const text = e.target.innerText.trim();
    filterEvents = events.filter(item => {
        const value = `/${item}`;
        return (value.startsWith(text) || text === '/') && text
    }).map(item => {
        return {
            label: `/${item} ${eventsMap[item] || ''}`,
            id: `/${item}`
        }
    })
    vscodeApi.postMessage({
        type: 'input.change',
        values: { data: filterEvents }
    })
}
function handleUpdatePlaceholder () {
    const $placeHolder = document.querySelector('.inscode-ai-optimizate-textarea-placeholder');
    const input = $textarea.innerText;
    if (input !== '') {
        $placeHolder.style.display = 'none';
    } else {
        $placeHolder.style.display = 'block';
    }
}
function handleChange (e) {
    handleUpdatePlaceholder();
    handleUpdateGenerateStatus();
    handleUpdateInput(e);
}
function handleKeyUp(e) {
    if (isOnComposition) return;
    const text = e.target.innerText;
    if (e.keyCode !== 8) {
        handleUpdateTextareaTag(text);
        return;
    }
    if (e.keyCode === 8 && findPrefixTag(text)) {
        $textarea.innerHTML = `${text}`;
    }
}
const UP = 'ArrowUp'
const DOWN = 'ArrowDown'
function handleToggleHistoryChat (e) {
    const { key } = e
    if (key === UP) {
        let index = window.ChatHistoryIndex + 1
        let historyItem = window.ChatHistory[index];
        if (!historyItem) {
            const firstHistoryItem = window.ChatHistory[0]
            if (!firstHistoryItem) {
                return;
            }
            index = 0;
            historyItem = firstHistoryItem;
        }
        $textarea.innerHTML = historyItem;
        window.ChatHistoryIndex = index;
        handleChange(e)
        handleFocusLast()
    } else if (key === DOWN) {
        let index = window.ChatHistoryIndex - 1;
        index = index < 0 ? window.ChatHistory.length - 1 : index;
        let historyItem = window.ChatHistory[index];
        if (!historyItem) return;
        $textarea.innerHTML = historyItem;
        window.ChatHistoryIndex = index;
        handleChange(e)
        handleFocusLast()
    }
}
function handleDown (e) {
    if ((e.metaKey || e.ctrlKey) && e.key === UP || e.key === DOWN) {
        handleToggleHistoryChat(e)
        return
    }
    if (e.shiftKey && e.key === 'Enter') {
        handleFocusLast();
        return;
    }
    if (isOnComposition) return;
    switch (e.keyCode) {
        case 27 :
            handleClose();
            break;
        case 38 :
            e.preventDefault();
            break;
        // 回车
        case 13 :
            if (filterEvents.length === 0) {
                handleRemoveEntry();
                handleSubmit();
            }
            e.preventDefault();
            break;
    }
    storeTextareaHtml();
}
function handeComposition(e) {
    if (e.type === 'compositionstart') {
        isOnComposition = true;
    } else if (e.type === 'compositionend') {
        isOnComposition = false;  
        handleKeyUp(e); 
    }
}
function handleActionClick (e) {
    const type = e.target.getAttribute('data-type');
    const value = $textarea.innerText;
    const query = getSubmitPrefix(value)
    if (!type) return;
    switch (type) {
        case 'continue':
            storeTextareaHtml();
            handleSubmit();
            break;
        case 'refuse':
            vscodeApi.postMessage({ type: 'inscode.ai.editor.optimizate.dialog.refuse', values: { type: query.type} });
            break;
        case 'accept':
            vscodeApi.postMessage({ type: 'inscode.ai.editor.optimizate.dialog.accept', values: { type: query.type } });
            break;
        case 'generate':
            if (loading) {
                vscodeApi.postMessage({ type: 'inscode.ai.editor.optimizate.dialog.abort' })
                handleFocusLast();
            } else {
                storeTextareaHtml();
                handleSubmit();
            }
            break;
    }
}
function init () {
    const params = window.ChatParams;
    // if (params && params.type && params.type !== 'chat' && params.code !== '') {
    if (params && params.type && params.type !== 'chat' ) {
        handleUpdateTextareaTag(`/${params.type}`)
        storeTextareaHtml();
        vscodeApi.postMessage({ type: 'inscode.ai.editor.optimizate.dialog.submit', values: { type: params.type }})
    }
    handleUpdateGenerateStatus();
}
const $textarea = document.querySelector('.inscode-ai-optimizate-textarea');
const $textareaHide = document.querySelector('.inscode-ai-optimizate-textarea-hidden');
const $close = document.querySelector('.inscode-ai-optimizate-close');
$textarea.addEventListener('input', handleChange)
$textarea.addEventListener('keydown', handleDown)
$textarea.addEventListener('keyup', handleKeyUp)
$textarea.addEventListener('compositionstart', handeComposition)
$textarea.addEventListener('compositionupdate', handeComposition)
$textarea.addEventListener('compositionend', handeComposition)
$close.addEventListener('click', handleClose)

window.addEventListener('message', (e) => {
    const data = e.data;
    if (data.type === 'inscode.ai.editor.optimizate.dialog.loading') {
        handleLoading(data.values);
    } else if (data.type === 'inscode.ai.editor.optimizate.dialog.tag') {
        handleUpdateTextareaTag(data.values.data)
        storeTextareaHtml();
        filterEvents = []
    }
    handleUpdatePlaceholder();
})
$textarea.focus();
const $actions = document.querySelector('.inscode-ai-optimizate-footer-r');
$actions.addEventListener('click', handleActionClick);

init();