const path = require('path');
const { VueLoaderPlugin } = require('vue-loader');
const Dotenv = require('dotenv-webpack');

module.exports = (env,argv) => {
  const envFile=`./env/.env.${env.platform?env.platform:'codearts'}.${argv.mode}`;
  // console.log('process.env:',process.env)
  return {
    target: 'web',
    entry: {
      'project': './src/client/project-entry.ts',
      'dialog': './src/client/dialog-entry.ts',
      'api': './src/client/api-entry.ts',
      'api-detail': './src/client/api-detail-entry.ts',
      'api-manage': './src/client/api-manage-entry.ts',
      'doc': './src/client/doc-entry.ts'
    },
    mode: argv.mode, // 通过 --mode=development | production(default) ,详见: https://www.webpackjs.com/configuration/mode/
    // devtool: argv.mode==='development' ? 'source-map' : 'nosources-source-map',
    devtool: false,
    output: {
      path: path.resolve(__dirname, 'resources/webview'),
      filename: '[name].js',
    },
    resolve: {
      extensions: ['*', '.js', '.jsx', '.vue', '.ts', '.tsx', '.json'],
      alias: {
        '@': path.resolve(__dirname, 'src')
      }
    },
    optimization: {
      splitChunks: {
        chunks: 'all', // 对所有模块都进行分割
        minSize: 20000, // 生成 chunk 的最小体积
        minChunks: 1, // 最少被引用次数
        maxAsyncRequests: 30, // 按需加载时的最大并行请求数
        maxInitialRequests: 30, // 入口点的最大并行请求数
        cacheGroups: {
          vendors: {
            test: /[\\/]node_modules[\\/]/,
            priority: -10,
            reuseExistingChunk: true,
            name: 'vendors'
          },
          common: {
            minChunks: 2, // 至少被两个 chunk 引用
            priority: -20,
            reuseExistingChunk: true,
            name: 'app'
          }
        }
      }
    },  
    module: {
      rules: [
        {
          test: /\.tsx?$/,
          loader: 'ts-loader',
          exclude: /node_modules/,
          options: {
            appendTsSuffixTo: [/\.vue$/],
            transpileOnly: true,
          },
        },
        {
          test: /\.(css|scss)$/,
          use: ['style-loader', 'css-loader', 'postcss-loader', 'sass-loader'],
        },
        {
          test: /\.vue$/,
          exclude: /node_modules/,
          use: [
            {
              loader: 'vue-loader',
            },
          ],
        },
        {
          test: /\.(png|jpe?g|gif|svg)$/i,
          use: [
            {
              loader: 'file-loader',
            },
          ],
        }
      ]
    },
    plugins: [
      // make sure to include the plugin!
      new VueLoaderPlugin(),
      // 应用 dotenv-webpack 插件
      new Dotenv({
        path: envFile, // 指定 .env 文件的位置，默认是项目根目录下的 .env 文件
        systemvars: true, // 将系统环境变量也引入
      }),
    ]
  }
};
